import logging

from django.apps import AppConfig

from .callhome import callhome_version

log = logging.getLogger(__name__)


class JfkDjangoCoreConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "jfk_django_core"
    verbose_name = "JFK Django Core"

    def ready(self) -> None:
        try:
            log.debug("Starting Callhome")
            callhome_version()
        except Exception:
            log.exception("JFK Django Core ready exception")



import logging
import time
from datetime import timedelta

from celery.exceptions import TimeoutError
from django.utils import timezone
from drf_spectacular.utils import OpenApiTypes, extend_schema
from rest_framework import permissions, status
from rest_framework.response import Response
from rest_framework.views import APIView

from jfk_django_core.models import CoreHealthcheck
from jfk_django_core.tasks import celery_healthcheck_task

log = logging.getLogger(__name__)


class HealthCheck(APIView):
    permission_classes = [permissions.AllowAny]
    authentication_classes = []

    @extend_schema(
        operation_id="healthcheck",
        request=OpenApiTypes.OBJECT,
    )
    def head(self, request, format=None) -> Response:
        try:
            return Response(status=status.HTTP_200_OK)
        except Exception:
            log.exception("HealthCheck Exception")
            return Response(
                "Unknown Error",
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )


class CeleryHealthCheck(APIView):
    permission_classes = [permissions.AllowAny]
    authentication_classes = []

    @extend_schema(
        operation_id="celery_healthcheck",
    )
    def head(self, request, format=None) -> Response:
        try:
            task_result = celery_healthcheck_task.apply_async()
            time.sleep(5)
            task_result.get(timeout=1)
            healt_check = CoreHealthcheck.objects.first()
            if (
                healt_check is not None
                and healt_check.healthcheck_run_datetime is not None
                and healt_check.healthcheck_run_datetime
                > timezone.now() - timedelta(seconds=10)
                and task_result.status == "SUCCESS"
            ):
                log.info("Celery Healthcheck Task Executed")
                return Response(status=status.HTTP_200_OK)
            return Response(
                "Celery is not executing tasks",
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )
        except TimeoutError:
            log.exception("Timeout Excheption on Healthcheck.")
            return Response(
                "Timeout on Celery Healthcheck Task",
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )
        except Exception:
            log.exception("HealthCheck Exception")
            return Response(
                "Unknown Error",
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )



from allauth.account.adapter import DefaultAccountAdapter


class NoNewUsersAccountAdapter(DefaultAccountAdapter):
    def is_open_for_signup(self, request):
        """Checks whether or not the site is open for signups.

        Next to simply returning True/False you can also intervene the
        regular flow by raising an ImmediateHttpResponse

        (Comment reproduced from the overridden method.)
        """
        return False



from rest_framework.authentication import BasicAuthentication

class BasicLikeAuthentication(BasicAuthentication):
    def authenticate_header(self, request):
        return f'BasicLike realm="{self.www_authenticate_realm}"'


"""JFK-Django-Core Base Apis."""

from knox.auth import TokenAuthentication
from rest_framework import permissions
from rest_framework.authentication import BasicAuthentication, SessionAuthentication
from rest_framework.views import APIView


class BaseAPIView(APIView):
    """Custom API View for all API Views in this file."""

    permission_classes = [permissions.IsAuthenticated]
    authentication_classes = [SessionAuthentication, TokenAuthentication, BasicAuthentication]



import uuid

from django.db import models


class UuidModel(models.Model):
    """Base Model with uuid."""

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, unique=True, editable=False)

    class Meta:
        abstract = True

    def __str__(self) -> str:
        return f"{self.id}"



from django.db import models

from .core import UuidModel


class CoreHealthcheck(UuidModel):
    """Base Model with uuid."""

    healthcheck_run_datetime = models.DateTimeField(verbose_name="Healthcheck Run Datetime", blank=True, null=True)

    class Meta:
        verbose_name = "Core Healthcheck"
        verbose_name_plural = "Core Healthchecks"

    def __str__(self) -> str:
        return f"{self.id}"



# Generated by Django 5.1.5 on 2025-01-26 19:50

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CoreHealthcheck',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True)),
                ('healthcheck_run_datetime', models.DateTimeField(blank=True, null=True, verbose_name='Healthcheck Run Datetime')),
            ],
            options={
                'verbose_name': 'Core Healthcheck',
                'verbose_name_plural': 'Core Healthchecks',
            },
        ),
    ]



from drf_spectacular.utils import extend_schema, extend_schema_view
from knox.views import LoginView, LogoutAllView, LogoutView
from rest_framework.authentication import SessionAuthentication

from ..authentication import BasicLikeAuthentication


@extend_schema_view(
    post=extend_schema(operation_id="login"),
)
class TokenLoginView(LoginView):
    authentication_classes = [BasicLikeAuthentication, SessionAuthentication]
    # permission_classes = [AllowAny]


@extend_schema_view(
    post=extend_schema(operation_id="logout"),
)
class TokenLogoutView(LogoutView):
    pass


@extend_schema_view(
    post=extend_schema(operation_id="logoutAll"),
)
class TokenLogoutAllView(LogoutAllView):
    pass



import logging

from celery import shared_task
from django.utils import timezone

from jfk_django_core.models import CoreHealthcheck

log = logging.getLogger(__name__)


@shared_task(ignore_result=False)
def celery_healthcheck_task() -> None:
    core_healthcheck_qs = CoreHealthcheck.objects.all()
    if core_healthcheck_qs.count() == 0:
        CoreHealthcheck.objects.create(healthcheck_run_datetime=timezone.now())
    elif core_healthcheck_qs.count() == 1:
        core_healthcheck = core_healthcheck_qs.first()
        if core_healthcheck is not None:
            core_healthcheck.healthcheck_run_datetime = timezone.now()
            core_healthcheck.save()
    else:
        log.error(f"Found {core_healthcheck_qs.count()} CoreHealthcheck records. Expected 0 or 1.")
        core_healthcheck_qs.delete()
        CoreHealthcheck.objects.create(healthcheck_run_datetime=timezone.now())



from django.urls import include, path
from drf_spectacular.views import SpectacularAPIView, SpectacularRedocView, SpectacularSwaggerView
from rest_framework import routers

from . import apis, viewsets

app_name = "jfk-django-core"

router = routers.DefaultRouter()
router.register(f"{app_name}/user", viewsets.UserViewSet, basename="user")

urlpatterns = [
    # Knox
    path("api/auth/login/", apis.TokenLoginView.as_view(), name="login"),
    path("api/auth/logout/", apis.TokenLogoutView.as_view(), name="logout"),
    path("api/auth/logoutall/", apis.TokenLogoutAllView.as_view(), name="logoutall"),
    # Swagger UI
    path("api/schema/", SpectacularAPIView.as_view(), name="schema"),
    path("api/schema/swagger-ui/", SpectacularSwaggerView.as_view(), name="swagger-ui"),
    path("api/schema/redoc/", SpectacularRedocView.as_view(), name="redoc"),
    # Core
    path(f"api/{app_name}/healthcheck/", apis.HealthCheck.as_view(), name="healthcheck"),
    path(f"api/{app_name}/celery-healthcheck/", apis.CeleryHealthCheck.as_view(), name="celery-healthcheck"),
    # Allauth
    path("accounts/", include("allauth.urls")),
]
